@@ -15,7 +15,10 @@
 
 # Import APIs into API package:
 from annotator.api.health_check_api import HealthCheckApi
+from annotator.api.text_contact_annotation_api import TextContactAnnotationApi
+from annotator.api.text_covid_symptom_annotation_api import TextCovidSymptomAnnotationApi
 from annotator.api.text_date_annotation_api import TextDateAnnotationApi
+from annotator.api.text_id_annotation_api import TextIdAnnotationApi
 from annotator.api.text_person_name_annotation_api import TextPersonNameAnnotationApi
 from annotator.api.text_physical_address_annotation_api import TextPhysicalAddressAnnotationApi
 from annotator.api.tool_api import ToolApi
