@@ -1,13 +1,16 @@
 """NLP data node client that interacts with the SDK datanodeclient"""
-from typing import List, Iterator
+from typing import List, Iterator, Union
 
 import requests
 
 import datanode
-from datanode.api import annotation_store_api, annotation_api, note_api
-from datanode.models import Annotation, AnnotationStore
+from datanode.api import annotation_store_api, annotation_api, dataset_api, note_api
+from datanode.models import Annotation, AnnotationStore, Dataset, Note
 import annotator
-from annotator.api import (text_date_annotation_api,
+from annotator.api import (text_contact_annotation_api,
+                           text_covid_symptom_annotation_api,
+                           text_date_annotation_api,
+                           text_id_annotation_api,
                            text_person_name_annotation_api,
                            text_physical_address_annotation_api,
                            tool_api)
@@ -239,145 +242,277 @@ def list_annotations(host: str, dataset_id: str,
             offset += limit
 
 
-def _annotate_person(api_client, note: dict) -> dict:
+def _annotate_person_name(api_client, text_annotation_request: dict) -> dict:
     """Annotate notes with date
 
     Args:
         host: Data node host IP
-        note: Clinical note
+        text_annotation_request: Text person name annotation request
 
     Yields:
         Annotated notes
 
     Examples:
-        >>> example_note = {
+        >>> example_request = {
         >>>    "note": {
         >>>        "identifier": "note-1",
-        >>>        "note_type": "loinc:LP29684-5",
+        >>>        "type": "loinc:LP29684-5",
         >>>        "patient_id": "507f1f77bcf86cd799439011",
         >>>        "text": "On 12/26/2020, Ms. Chloe Price met with Dr. Prescott."
         >>>    }
         >>> }
-        >>> host = "0.0.0.0:8080/api/v1"
+        >>> host = "0.0.0.0/api/v1"
         >>> configuration = annotator.Configuration(host=host)
         >>> with annotator.ApiClient(configuration) as api_client:
-        >>>     annotations = _annotate_person(api_client=api_client,
-        >>>                                    note=example_note)
+        >>>     annotations = _annotate_person_name(
+        >>>         api_client=api_client,
+        >>>         text_annotation_request=example_request
+        >>>     )
 
     """
     # host = "http://10.23.55.45:9000/api/v1"
     api_instance = text_person_name_annotation_api.TextPersonNameAnnotationApi(api_client)
     annotations = api_instance.create_text_person_name_annotations(
-        text_person_name_annotation_request=note
+        text_person_name_annotation_request=text_annotation_request
     )
     return annotations
 
 
-def _annotate_address(api_client, note: dict) -> dict:
+def _annotate_physical_address(api_client, text_annotation_request: dict) -> dict:
     """Annotate notes with date
 
     Args:
         host: Data node host IP
-        note: Clinical note
+        text_annotation_request: Text physical address annotation request
 
     Yields:
         Annotated notes
 
     Examples:
-        >>> example_note = {
+        >>> example_request = {
         >>>    "note": {
         >>>        "identifier": "note-1",
-        >>>        "note_type": "loinc:LP29684-5",
+        >>>        "type": "loinc:LP29684-5",
         >>>        "patient_id": "507f1f77bcf86cd799439011",
         >>>        "text": "On 12/26/2020, Ms. Chloe Price met with Dr. Prescott."
         >>>    }
         >>> }
-        >>> host = "0.0.0.0:8080/api/v1"
+        >>> host = "0.0.0.0/api/v1"
         >>> configuration = annotator.Configuration(host=host)
         >>> with annotator.ApiClient(configuration) as api_client:
-        >>>     annotations = _annotate_address(api_client=api_client,
-        >>>                                     note=example_note)
+        >>>     annotations = _annotate_physical_address(
+        >>>         api_client=api_client,
+        >>>         text_annotation_request=example_request
+        >>>     )
 
     """
     # host = "http://10.23.55.45:9000/api/v1"
     api_instance = text_physical_address_annotation_api.TextPhysicalAddressAnnotationApi(api_client)
     annotations = api_instance.create_text_physical_address_annotations(
-        text_physical_address_annotation_request=note
+        text_physical_address_annotation_request=text_annotation_request
     )
     return annotations
 
 
-def _annotate_date(api_client, note: dict) -> dict:
+def _annotate_date(api_client, text_annotation_request: dict) -> dict:
     """Annotate notes with date
 
     Args:
         host: Data node host IP
-        note: Clinical note
+        text_annotation_request: Text date annotation request
 
     Yields:
         Annotated notes
 
     Examples:
-        >>> example_note = {
+        >>> example_request = {
         >>>    "note": {
         >>>        "identifier": "note-1",
-        >>>        "note_type": "loinc:LP29684-5",
+        >>>        "type": "loinc:LP29684-5",
         >>>        "patient_id": "507f1f77bcf86cd799439011",
         >>>        "text": "On 12/26/2020, Ms. Chloe Price met with Dr. Prescott."
         >>>    }
         >>> }
-        >>> host = "0.0.0.0:8080/api/v1"
+        >>> host = "0.0.0.0/api/v1"
         >>> configuration = annotator.Configuration(host=host)
         >>> with annotator.ApiClient(configuration) as api_client:
-        >>>     annotations = _annotate_date(api_client=api_client,
-        >>>                                  note=example_note)
+        >>>     annotations = _annotate_person(
+        >>>         api_client=api_client,
+        >>>         text_annotation_request=example_request
+        >>> )
 
     """
     # host = "http://10.23.55.45:9000/api/v1"
     api_instance = text_date_annotation_api.TextDateAnnotationApi(api_client)
     annotations = api_instance.create_text_date_annotations(
-        text_date_annotation_request=note
+        text_date_annotation_request=text_annotation_request
     )
     return annotations
 
 
-def annotate_note(host: str, note: dict, annotator_type: str) -> dict:
+def _annotate_contact(api_client, text_annotation_request: dict) -> dict:
+    """Annotate notes with contact
+
+    Args:
+        host: Data node host IP
+        text_annotation_request: Text contact annotation request
+
+    Yields:
+        Annotated notes
+
+    Examples:
+        >>> example_request = {
+        >>>    "note": {
+        >>>        "identifier": "note-1",
+        >>>        "type": "loinc:LP29684-5",
+        >>>        "patient_id": "507f1f77bcf86cd799439011",
+        >>>        "text": "On 12/26/2020, Ms. Chloe Price met with Dr. Prescott. Her phone number is 203-555-4545."
+        >>>    }
+        >>> }
+        >>> host = "0.0.0.0/api/v1"
+        >>> configuration = annotator.Configuration(host=host)
+        >>> with annotator.ApiClient(configuration) as api_client:
+        >>>     annotations = _annotate_contact(
+        >>>         api_client=api_client,
+        >>>         text_annotation_request=example_request
+        >>>     )
+
+    """
+    # host = "http://10.23.55.45:9000/api/v1"
+    api_instance = text_contact_annotation_api.TextContactAnnotationApi(
+        api_client
+    )
+    annotations = api_instance.create_text_contact_annotations(
+        text_contact_annotation_request=text_annotation_request
+    )
+    return annotations
+
+
+def _annotate_id(api_client, text_annotation_request: dict) -> dict:
+    """Annotate notes with id
+
+    Args:
+        host: Data node host IP
+        text_annotation_request: Text id annotation request
+
+    Yields:
+        Annotated notes
+
+    Examples:
+        >>> example_request = {
+        >>>    "note": {
+        >>>        "identifier": "note-1",
+        >>>        "type": "loinc:LP29684-5",
+        >>>        "patient_id": "507f1f77bcf86cd799439011",
+        >>>        "text": "On 12/26/2020, Ms. Chloe Price met with Dr. Prescott. Her SSN is 123-45-6789."
+        >>>    }
+        >>> }
+        >>> host = "0.0.0.0/api/v1"
+        >>> configuration = annotator.Configuration(host=host)
+        >>> with annotator.ApiClient(configuration) as api_client:
+        >>>     annotations = _annotate_id(
+        >>>         api_client=api_client,
+        >>>         text_annotation_request=example_request
+        >>>     )
+
+    """
+    # host = "http://10.23.55.45:9000/api/v1"
+    api_instance = text_id_annotation_api.TextIdAnnotationApi(
+        api_client
+    )
+    annotations = api_instance.create_text_id_annotations(
+        text_id_annotation_request=text_annotation_request
+    )
+    return annotations
+
+
+def _annotate_covid_symptom(api_client, text_annotation_request: dict) -> dict:
+    """Annotate notes with covid symptom
+
+    Args:
+        host: Data node host IP
+        text_annotation_request: Text id annotation request
+
+    Yields:
+        Annotated notes
+
+    Examples:
+        >>> example_request = {
+        >>>    "note": {
+        >>>        "identifier": "note-1",
+        >>>        "type": "loinc:LP29684-5",
+        >>>        "patient_id": "507f1f77bcf86cd799439011",
+        >>>        "text": "On 12/26/2020, Ms. Chloe Price met with Dr. Prescott. She's had a dry cough for almost a week now."
+        >>>    }
+        >>> }
+        >>> host = "0.0.0.0/api/v1"
+        >>> configuration = annotator.Configuration(host=host)
+        >>> with annotator.ApiClient(configuration) as api_client:
+        >>>     annotations = _annotate_covid_symptom(
+        >>>         api_client=api_client,
+        >>>         text_annotation_request=example_request
+        >>>     )
+
+    """  # noqa: E501
+    # host = "http://10.23.55.45:9000/api/v1"
+    api_instance = text_covid_symptom_annotation_api.TextCovidSymptomAnnotationApi(  # noqa: E501
+        api_client
+    )
+    annotations = api_instance.create_text_covid_symptom_annotations(
+        text_covid_symptom_annotation_request=text_annotation_request
+    )
+    return annotations
+
+
+def annotate_note(host: str, note: Union[dict, Note],
+                  tool_type: str) -> dict:
     """Annotate notes
 
     Args:
         host: Data node host IP
         note: Clinical note
-        annotator_type: Type of annotator
+        tool_type: Type of annotator
 
     Yields:
         Annotated notes
 
     Examples:
         >>> example_note = {
-        >>>    "note": {
-        >>>        "identifier": "note-1",
-        >>>        "noteType": "loinc:LP29684-5",
-        >>>        "patientId": "507f1f77bcf86cd799439011",
-        >>>        "text": "On 12/26/2020, Ms. Chloe Price met with Dr. Prescott."
-        >>>    }
+        >>>    "identifier": "note-1",
+        >>>    "noteType": "loinc:LP29684-5",
+        >>>    "patientId": "507f1f77bcf86cd799439011",
+        >>>    "text": "On 12/26/2020, Ms. Chloe Price met with Dr. Prescott."
         >>> }
         >>> annotations = annotate_note(host="0.0.0.0/api/v1",
         >>>                             note=example_note,
-        >>>                             annotator_type="date")
+        >>>                             tool_type="date")
 
     """
     # host = "http://10.23.55.45:9000/api/v1"
     configuration = annotator.Configuration(host=host)
-    new_note = utils.change_keys(note, utils.camelcase_to_snakecase)
+    # Change clinical note into text annotation request format
+    if isinstance(note, Note):
+        text_annotator_req = {"note": note.to_dict()}
+    else:
+        text_annotator_req = {
+            "note": utils.change_keys(note, utils.camelcase_to_snakecase)
+        }
     with annotator.ApiClient(configuration) as api_client:
-        if annotator_type == "date":
-            annotations = _annotate_date(api_client, new_note)
-        elif annotator_type == "person":
-            annotations = _annotate_person(api_client, new_note)
-        elif annotator_type == "address":
-            annotations = _annotate_address(api_client, new_note)
+        if tool_type == "nlpsandbox:date-annotator":
+            annotations = _annotate_date(api_client, text_annotator_req)
+        elif tool_type == "nlpsandbox:person-name-annotator":
+            annotations = _annotate_person_name(api_client, text_annotator_req)
+        elif tool_type == "nlpsandbox:physical-address-annotator":
+            annotations = _annotate_physical_address(api_client, text_annotator_req)
+        elif tool_type == "nlpsandbox:contact-annotator":
+            annotations = _annotate_contact(api_client, text_annotator_req)
+        elif tool_type == "nlpsandbox:id-annotator":
+            annotations = _annotate_id(api_client, text_annotator_req)
+        elif tool_type == "nlpsandbox:covid-symptom-annotator":
+            annotations = _annotate_covid_symptom(api_client,
+                                                  text_annotator_req)
         else:
-            raise ValueError(f"Invalid annotator_type: {annotator_type}")
+            raise ValueError(f"Invalid annotator_type: {tool_type}")
         sanitized_annotations = api_client.sanitize_for_serialization(
             annotations
         )
@@ -412,6 +547,41 @@ def get_tool(host: str) -> Tool:
     return tool_info
 
 
+def list_datasets(host: str) -> List[dict]:
+    """Get all datasets
+
+    Args:
+        host: Data node host IP
+
+    Yields:
+        list of datasets.
+
+    Examples:
+        >>> datasets = list_datasets(host="0.0.0.0/api/v1")
+        >>> list(datasets)[0]
+        >>> {
+        >>>    "name": "datasets/testing"
+        >>> }
+    """
+    configuration = datanode.Configuration(host=host)
+    offset = 0
+    limit = 10
+    with datanode.ApiClient(configuration) as api_client:
+        api_instance = dataset_api.DatasetApi(api_client)
+        # Obtain all clinical notes
+        next_page = True
+        while next_page:
+            datasets = api_instance.list_datasets(offset=offset, limit=limit)
+            # change from snake case to camel case
+            sanitized = api_client.sanitize_for_serialization(
+                datasets.datasets
+            )
+            for dataset in sanitized:
+                yield dataset
+            next_page = datasets.links.next
+            offset += limit
+
+
 def _get_tool_redirect(host: str) -> Tool:
     """Make sure tool's base URL redirects to tool service
 
@@ -486,3 +656,25 @@ def store_annotations(host: str, dataset_id: str, annotation_store_id: str,
             annotation_id=annotation_id,
             annotation=annotation
         )
+
+
+def store_dataset(host: str, dataset_id: str) -> Dataset:
+    """Creates a dataset"""
+    configuration = datanode.Configuration(host=host)
+    # Enter a context with an instance of the API client
+    with datanode.ApiClient(configuration) as api_client:
+        # Create an instance of the API class
+        api_instance = dataset_api.DatasetApi(api_client)
+        body = {}
+        dataset = api_instance.create_dataset(dataset_id, body=body)
+    return dataset
+
+
+def delete_dataset(host: str, dataset_id: str):
+    """Deletes a dataset"""
+    configuration = datanode.Configuration(host=host)
+    # Enter a context with an instance of the API client
+    with datanode.ApiClient(configuration) as api_client:
+        # Create an instance of the API class
+        api_instance = dataset_api.DatasetApi(api_client)
+        api_instance.delete_dataset(dataset_id)
